package com.cultural.heritage.service.order.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.constant.CommonConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.mapper.*;
import com.cultural.heritage.model.dto.order.OrderAddRequest;
import com.cultural.heritage.model.dto.order.OrderItemAddRequest;
import com.cultural.heritage.model.dto.order.OrderQueryRequest;
import com.cultural.heritage.model.dto.order.capital.OrderItemMainInfoAddRequest;
import com.cultural.heritage.model.dto.order.capital.OrderMainInfoAddRequest;
import com.cultural.heritage.model.dto.snapshot.AddressSnapshot;
import com.cultural.heritage.model.dto.snapshot.ContactsSnapshot;
import com.cultural.heritage.model.dto.snapshot.CouponSnapshot;
import com.cultural.heritage.model.dto.snapshot.GoodSnapshot;
import com.cultural.heritage.model.entity.*;
import com.cultural.heritage.service.order.OrderService;
import com.cultural.heritage.utils.SqlUtils;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

@Service
public class OrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements OrderService {


    @Resource
    private AddressMapper addressMapper;


    @Resource
    private ContactsMapper contactsMapper;


    @Resource
    private CouponMapper couponMapper;


    @Resource
    private GoodMapper goodMapper;


    /**
     * 获取查询条件
     */
    @Override
    public QueryWrapper<Order> getQueryWrapper(OrderQueryRequest orderQueryRequest) {
        if (orderQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long userId = orderQueryRequest.getUserId();
        Long orderId = orderQueryRequest.getOrderId();
        Double minTotalAmount = orderQueryRequest.getMinTotalAmount();
        Double maxTotalAmount = orderQueryRequest.getMaxTotalAmount();
        String orderStatus = orderQueryRequest.getOrderStatus();
        String orderNumber = orderQueryRequest.getOrderNumber();
        Date startTime = orderQueryRequest.getStartTime();
        Date endTime = orderQueryRequest.getEndTime();
        String sortOrder = orderQueryRequest.getSortOrder();
        String sortField = orderQueryRequest.getSortField();
        QueryWrapper<Order> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ObjectUtils.isNotEmpty(userId), "userId", userId);
        queryWrapper.eq(ObjectUtils.isNotEmpty(orderId), "id", orderId);
        queryWrapper.ge(ObjectUtils.isNotEmpty(minTotalAmount), "totalAmount", minTotalAmount);
        queryWrapper.le(ObjectUtils.isNotEmpty(maxTotalAmount), "totalAmount", maxTotalAmount);
        queryWrapper.eq(StringUtils.isNotBlank(orderStatus), "orderStatus", orderStatus);
        queryWrapper.between(startTime != null && endTime != null, "createTime", startTime, endTime);
        queryWrapper.eq(StringUtils.isNotBlank(orderNumber), "orderNumber", orderNumber);
        queryWrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC),
                sortField);
        return queryWrapper;
    }


    @Override
    public OrderAddRequest initOrderMainInfo(OrderMainInfoAddRequest orderMainInfoAddRequest) {
        Long userId = orderMainInfoAddRequest.getUserId();
        String userName = orderMainInfoAddRequest.getUserName();
        Long addressId = orderMainInfoAddRequest.getAddressId();
        Long contactsId = orderMainInfoAddRequest.getContactsId();
        Long couponId = orderMainInfoAddRequest.getCouponId();
        String orderNumber = orderMainInfoAddRequest.getOrderNumber();
        String orderStatus = orderMainInfoAddRequest.getOrderStatus();
        Double totalAmount = orderMainInfoAddRequest.getTotalAmount();

        List<OrderItemMainInfoAddRequest> orderItemMainInfoAddRequestList = orderMainInfoAddRequest.getOrderItemMainInfoAddRequestList();

        OrderAddRequest orderAddRequest = new OrderAddRequest();

        orderAddRequest.setUserId(userId);
        orderAddRequest.setUserName(userName);
        orderAddRequest.setOrderNumber(orderNumber);
        orderAddRequest.setOrderStatus(orderStatus);
        orderAddRequest.setTotalAmount(totalAmount);

        // 获取订单地址信息
        QueryWrapper<Address> addressQueryWrapper = new QueryWrapper<>();
        addressQueryWrapper.eq("id", addressId);
        Address address = addressMapper.selectOne(addressQueryWrapper);
        ThrowUtils.throwIf(address == null, ErrorCode.NOT_FOUND_ERROR);
        AddressSnapshot addressSnapshot = new AddressSnapshot();
        BeanUtils.copyProperties(address, addressSnapshot);
        // 获取订单联系人信息
        QueryWrapper<Contacts> contactsQueryWrapper = new QueryWrapper<>();
        contactsQueryWrapper.eq("id", contactsId);
        Contacts contacts = contactsMapper.selectOne(contactsQueryWrapper);
        ThrowUtils.throwIf(contacts == null, ErrorCode.NOT_FOUND_ERROR);
        ContactsSnapshot contactsSnapshot = new ContactsSnapshot();
        BeanUtils.copyProperties(contacts, contactsSnapshot);
        // 获取订单优惠券信息
        QueryWrapper<Coupon> couponQueryWrapper = new QueryWrapper<>();
        couponQueryWrapper.eq("id", couponId);
        Coupon coupon = couponMapper.selectOne(couponQueryWrapper);
        ThrowUtils.throwIf(coupon == null, ErrorCode.NOT_FOUND_ERROR);
        CouponSnapshot couponSnapshot = new CouponSnapshot();
        BeanUtils.copyProperties(coupon, couponSnapshot);

        orderAddRequest.setAddressSnapshot(addressSnapshot);
        orderAddRequest.setContactsSnapshot(contactsSnapshot);
        orderAddRequest.setCouponSnapshot(couponSnapshot);


        List<OrderItemAddRequest> orderItemAddRequestList = orderItemMainInfoAddRequestList.stream().map(orderItemMainInfoAddRequest -> {
            Long goodId = orderItemMainInfoAddRequest.getGoodId();
            Integer quantity = orderItemMainInfoAddRequest.getQuantity();
            OrderItemAddRequest orderItemAddRequest = new OrderItemAddRequest();
            QueryWrapper<Good> goodQueryWrapper = new QueryWrapper<>();
            goodQueryWrapper.eq("id", goodId);
            Good good = goodMapper.selectOne(goodQueryWrapper);
            ThrowUtils.throwIf(good == null, ErrorCode.NOT_FOUND_ERROR);
            GoodSnapshot goodSnapshot = new GoodSnapshot();
            BeanUtils.copyProperties(good, goodSnapshot);
            orderItemAddRequest.setGoodSnapshot(goodSnapshot);
            orderItemAddRequest.setPriceSnapshot(good.getPrice());
            orderItemAddRequest.setQuantity(quantity);
            orderItemAddRequest.setItemTotalAmount(good.getPrice() * quantity);
            return orderItemAddRequest;
        }).toList();

        orderAddRequest.setOrderItemList(orderItemAddRequestList);

        return orderAddRequest;
    }




}
